/***************************************************************************
* Project:  OpenCPN
* Purpose:  DR
* Author:   Jon Gough
*
* Project:  OpenCPN
*
***************************************************************************
*   Copyright (C) 2013 by David S. Register                               *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,  USA.         *
**************************************************************************/

#ifndef __ODNEWODPOINTDIALOGIMPL_H__
#define __ODNEWODPOINTDIALOGIMPL_H__

/**
 * @file
 * Subclass of ODNewPointDialogDef, which is generated by wxFormBuilder.
 */

#include "ODNewODPointDialogDef.h"

//    Constants for buttons
enum
{
    ID_ODNEWPOINTDIALOGBUTTON_BOUNDARY = 0,
    ID_ODNEWPOINTDIALOGBUTTON_TEXT,
    
    ID_ODNEWPOINTDIALOGBUTTON_LAST
};
    

///////////////////////////////////////////////////////////////////////////////
/// Class NewODPoint
///////////////////////////////////////////////////////////////////////////////
class ODNewODPointDialogImpl : public ODNewODPointDialogDef 
{
	private:

	protected:
        // Handlers for ODNewODPointDialogDef events.
        void OnOK( wxCommandEvent& event );
        void OnCancel( wxCommandEvent& event );
        
	public:
        ODNewODPointDialogImpl( wxWindow *parent );
        
        bool m_bOK;
        int  m_iSelection;
        
	
};

#endif //__ODNEWODPOINTDIALOGIMPL_H__
