/***************************************************************************
 *
 * Project:  OpenCPN
 * Purpose:  OCPN Draw Import Boundary support
 * Author:   Jon Gough
 *
 ***************************************************************************
 *   Copyright (C) 2010 by David S. Register                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,  USA.         *
 **************************************************************************/
 
#ifndef BOUNDARYCSVIMPORT_H
#define BOUNDARYCSVIMPORT_H

class wxStringTokenizer;

class BoundaryCSVImport 
{
    public:
        BoundaryCSVImport();
        BoundaryCSVImport(wxStringTokenizer BoundaryCSV);
        virtual ~BoundaryCSVImport();

        wxString    m_sName;
        bool        m_bExclusion;
        bool        m_bInclusion;
        bool        m_bVisible;
        wxColour    m_LineColour;
        wxColour    m_FillColour;
        bool        m_bIsBeingCreated;
        
    protected:
    private:

};
#endif // BOUNDARYCSVIMPORT_H
