#include "ODToolbarImpl.h"

ODToolbarImpl::ODToolbarImpl( wxWindow* parent )
:
ODToolbarDialog( parent )
{

}

void ODToolbarImpl::OnToolClicked( wxCommandEvent& event )
{
// TODO: Implement OnToolClicked
}
