#include "ODDRDialogImpl.h"

ODDRDialogImpl::ODDRDialogImpl( wxWindow* parent )
:
MyDialogDR( parent )
{

}

void ODDRDialogImpl::OnOK( wxCommandEvent& event )
{
// TODO: Implement OnOK
}

void ODDRDialogImpl::OnCancel( wxCommandEvent& event )
{
// TODO: Implement OnCancel
}
