#ifndef __ODToolbarImpl__
#define __ODToolbarImpl__

/**
@file
Subclass of ODToolbarDialog, which is generated by wxFormBuilder.
*/

#include "ODToolbarDef.h"

//// end generated include

/** Implementing ODToolbarDialog */
class ODToolbarImpl : public ODToolbarDialog
{
	protected:
		// Handlers for ODToolbarDialog events.
		void OnToolClicked( wxCommandEvent& event );
	public:
		/** Constructor */
		ODToolbarImpl( wxWindow* parent );
	//// end generated class members
	
};

#endif // __ODToolbarImpl__
