#ifndef __ODPathPropertiesDialogImpl__
#define __ODPathPropertiesDialogImpl__

/**
@file
Subclass of ODPathPropertiesDialogDef, which is generated by wxFormBuilder.
*/

#include "ODPathPropertiesDialogDef.h"

//// end generated include

/** Implementing ODPathPropertiesDialogDef */
class ODPathPropertiesDialogImpl : public ODPathPropertiesDialogDef
{
	protected:
		// Handlers for ODPathPropertiesDialogDef events.
		void OnClose( wxCloseEvent& event );
		void OnOK( wxCommandEvent& event );
		void OnCancel( wxCommandEvent& event );
	public:
		/** Constructor */
		ODPathPropertiesDialogImpl( wxWindow* parent );
	//// end generated class members






};

#endif // __ODPathPropertiesDialogImpl__
