#include "ODPointPropertiesImpl.h"

ODPointPropertiesImpl::ODPointPropertiesImpl( wxWindow* parent )
:
ODPointPropertiesDialog( parent )
{

}

void ODPointPropertiesImpl::OnRightClick( wxMouseEvent& event )
{
	// TODO: Implement OnRightClick
}

void ODPointPropertiesImpl::OnPositionCtlUpdated( wxCommandEvent& event )
{
	// TODO: Implement OnPositionCtlUpdated
}

void ODPointPropertiesImpl::OnRightClick( wxMouseEvent& event )
{
	// TODO: Implement OnRightClick
}

void ODPointPropertiesImpl::OnArrivalRadiusChange( wxCommandEvent& event )
{
	// TODO: Implement OnArrivalRadiusChange
}

void ODPointPropertiesImpl::OnShowRangeRingsSelect( wxCommandEvent& event )
{
	// TODO: Implement OnShowRangeRingsSelect
}

void ODPointPropertiesImpl::OnRangeRingsStepChange( wxCommandEvent& event )
{
	// TODO: Implement OnRangeRingsStepChange
}

void ODPointPropertiesImpl::OnDescChangedBasic( wxCommandEvent& event )
{
	// TODO: Implement OnDescChangedBasic
}

void ODPointPropertiesImpl::OnExtDescriptionClick( wxCommandEvent& event )
{
	// TODO: Implement OnExtDescriptionClick
}

void ODPointPropertiesImpl::OnPointPropertiesOKClick( wxCommandEvent& event )
{
	// TODO: Implement OnPointPropertiesOKClick
}

void ODPointPropertiesImpl::OnPointPropertiesCancelClick( wxCommandEvent& event )
{
	// TODO: Implement OnPointPropertiesCancelClick
}
