#include "ODPropertiesDialogImpl.h"

ODPropertiesDialogImpl::ODPropertiesDialogImpl( wxWindow* parent )
:
ODPropertiesDialogDef( parent )
{

}

void ODPropertiesDialogImpl::OnDrawPropertiesOKClick( wxCommandEvent& event )
{
// TODO: Implement OnDrawPropertiesOKClick
}

void ODPropertiesDialogImpl::OnDrawPropertiesCancelClick( wxCommandEvent& event )
{
// TODO: Implement OnDrawPropertiesCancelClick
}

void ODPropertiesDialogImpl::OnDrawPropertiesApplyClick( wxCommandEvent& event )
{
// TODO: Implement OnDrawPropertiesApplyClick
}
