#ifndef __ODLinkPropertiesDialogImpl__
#define __ODLinkPropertiesDialogImpl__

/**
@file
Subclass of ODLinkPropertiesDialogDef, which is generated by wxFormBuilder.
*/

#include "ODLinkPropertiesDialogDef.h"

//// end generated include

/** Implementing ODLinkPropertiesDialogDef */
class ODLinkPropertiesDialogImpl : public ODLinkPropertiesDialogDef
{
	public:
		/** Constructor */
		ODLinkPropertiesDialogImpl( wxWindow* parent );
	//// end generated class members

};

#endif // __ODLinkPropertiesDialogImpl__
