#include "PathAndPointManagerDialogImpl.h"

PathAndPointManagerDialogImpl::PathAndPointManagerDialogImpl( wxWindow* parent )
:
PathAndPointManagerDialogDef( parent )
{

}

void PathAndPointManagerDialogImpl::OnTabSwitch( wxNotebookEvent& event )
{
// TODO: Implement OnTabSwitch
}

void PathAndPointManagerDialogImpl::OnPathToggleVisibility( wxMouseEvent& event )
{
// TODO: Implement OnPathToggleVisibility
}

void PathAndPointManagerDialogImpl::OnPathColumnClicked( wxListEvent& event )
{
// TODO: Implement OnPathColumnClicked
}

void PathAndPointManagerDialogImpl::OnPathDefaultAction( wxListEvent& event )
{
// TODO: Implement OnPathDefaultAction
}

void PathAndPointManagerDialogImpl::OnPathDeSelected( wxListEvent& event )
{
// TODO: Implement OnPathDeSelected
}

void PathAndPointManagerDialogImpl::OnPathRightClick( wxListEvent& event )
{
// TODO: Implement OnPathRightClick
}

void PathAndPointManagerDialogImpl::OnPathSelected( wxListEvent& event )
{
// TODO: Implement OnPathSelected
}

void PathAndPointManagerDialogImpl::OnPathPropertiesClick( wxCommandEvent& event )
{
// TODO: Implement OnPathPropertiesClick
}

void PathAndPointManagerDialogImpl::OnPathActivateClick( wxCommandEvent& event )
{
// TODO: Implement OnPathActivateClick
}

void PathAndPointManagerDialogImpl::OnPathButtonLeftDown( wxMouseEvent& event )
{
// TODO: Implement OnPathButtonLeftDown
}

void PathAndPointManagerDialogImpl::OnPathCenterViewClick( wxCommandEvent& event )
{
// TODO: Implement OnPathCenterViewClick
}

void PathAndPointManagerDialogImpl::OnPathDeleteClick( wxCommandEvent& event )
{
// TODO: Implement OnPathDeleteClick
}

void PathAndPointManagerDialogImpl::OnPathExportSelectedClick( wxCommandEvent& event )
{
// TODO: Implement OnPathExportSelectedClick
}

void PathAndPointManagerDialogImpl::OnPathDeleteAllClick( wxCommandEvent& event )
{
// TODO: Implement OnPathDeleteAllClick
}

void PathAndPointManagerDialogImpl::OnODPointToggleVisibility( wxMouseEvent& event )
{
// TODO: Implement OnODPointToggleVisibility
}

void PathAndPointManagerDialogImpl::OnODPointColumnClick( wxListEvent& event )
{
// TODO: Implement OnODPointColumnClick
}

void PathAndPointManagerDialogImpl::OnODPointDefaultAction( wxListEvent& event )
{
// TODO: Implement OnODPointDefaultAction
}

void PathAndPointManagerDialogImpl::OnODPointDeSelected( wxListEvent& event )
{
// TODO: Implement OnODPointDeSelected
}

void PathAndPointManagerDialogImpl::OnODPointRightClick( wxListEvent& event )
{
// TODO: Implement OnODPointRightClick
}

void PathAndPointManagerDialogImpl::OnODPointSelected( wxListEvent& event )
{
// TODO: Implement OnODPointSelected
}

void PathAndPointManagerDialogImpl::OnODPointNewClick( wxCommandEvent& event )
{
// TODO: Implement OnODPointNewClick
}

void PathAndPointManagerDialogImpl::OnODPointPropertiesClick( wxCommandEvent& event )
{
// TODO: Implement OnODPointPropertiesClick
}

void PathAndPointManagerDialogImpl::OnODPointActivateClick( wxCommandEvent& event )
{
// TODO: Implement OnODPointActivateClick
}

void PathAndPointManagerDialogImpl::OnODPointCenterViewClick( wxCommandEvent& event )
{
// TODO: Implement OnODPointCenterViewClick
}

void PathAndPointManagerDialogImpl::OnODPointDeleteClick( wxCommandEvent& event )
{
// TODO: Implement OnODPointDeleteClick
}

void PathAndPointManagerDialogImpl::OnODPointExportSecectedClick( wxCommandEvent& event )
{
// TODO: Implement OnODPointExportSecectedClick
}

void PathAndPointManagerDialogImpl::OnODPointDeleteAllClick( wxCommandEvent& event )
{
// TODO: Implement OnODPointDeleteAllClick
}

void PathAndPointManagerDialogImpl::OnLayerToggleVisibility( wxMouseEvent& event )
{
// TODO: Implement OnLayerToggleVisibility
}

void PathAndPointManagerDialogImpl::OnLayerColumnClick( wxListEvent& event )
{
// TODO: Implement OnLayerColumnClick
}

void PathAndPointManagerDialogImpl::OnLayerDefaultAction( wxListEvent& event )
{
// TODO: Implement OnLayerDefaultAction
}

void PathAndPointManagerDialogImpl::OnLayerDeSelected( wxListEvent& event )
{
// TODO: Implement OnLayerDeSelected
}

void PathAndPointManagerDialogImpl::OnLayerRightClick( wxListEvent& event )
{
// TODO: Implement OnLayerRightClick
}

void PathAndPointManagerDialogImpl::OnLayerSelected( wxListEvent& event )
{
// TODO: Implement OnLayerSelected
}

void PathAndPointManagerDialogImpl::OnLayerTemporaryClick( wxCommandEvent& event )
{
// TODO: Implement OnLayerTemporaryClick
}

void PathAndPointManagerDialogImpl::OnLayerDeleteClick( wxCommandEvent& event )
{
// TODO: Implement OnLayerDeleteClick
}

void PathAndPointManagerDialogImpl::OnLayerShowOnChartClick( wxCommandEvent& event )
{
// TODO: Implement OnLayerShowOnChartClick
}

void PathAndPointManagerDialogImpl::OnLayerShowPointNamesClick( wxCommandEvent& event )
{
// TODO: Implement OnLayerShowPointNamesClick
}

void PathAndPointManagerDialogImpl::OnLayerListContentsClick( wxCommandEvent& event )
{
// TODO: Implement OnLayerListContentsClick
}

void PathAndPointManagerDialogImpl::OnImportClick( wxCommandEvent& event )
{
// TODO: Implement OnImportClick
}

void PathAndPointManagerDialogImpl::OnExportAllClick( wxCommandEvent& event )
{
// TODO: Implement OnExportAllClick
}

void PathAndPointManagerDialogImpl::OnOKClick( wxCommandEvent& event )
{
// TODO: Implement OnOKClick
}


void PathAndPointManagerDialogImpl::OnPointDeleteClick( wxCommandEvent& event )
{
// TODO: Implement OnPointDeleteClick
}

void PathAndPointManagerDialogImpl::OnPointCenterViewClick( wxCommandEvent& event )
{
// TODO: Implement OnPointCenterViewClick
}

void PathAndPointManagerDialogImpl::OnPointToggleVisibility( wxMouseEvent& event )
{
// TODO: Implement OnPointToggleVisibility
}

void PathAndPointManagerDialogImpl::OnPointColumnClick( wxListEvent& event )
{
// TODO: Implement OnPointColumnClick
}

void PathAndPointManagerDialogImpl::OnPointRightClick( wxListEvent& event )
{
// TODO: Implement OnPointRightClick
}

void PathAndPointManagerDialogImpl::OnPointExportSecectedClick( wxCommandEvent& event )
{
// TODO: Implement OnPointExportSecectedClick
}

void PathAndPointManagerDialogImpl::OnPointSelected( wxListEvent& event )
{
// TODO: Implement OnPointSelected
}

void PathAndPointManagerDialogImpl::OnTabSwitched( wxNotebookEvent& event )
{
// TODO: Implement OnTabSwitched
}

void PathAndPointManagerDialogImpl::OnPointActivateClick( wxCommandEvent& event )
{
// TODO: Implement OnPointActivateClick
}

void PathAndPointManagerDialogImpl::OnPointDeleteAllClick( wxCommandEvent& event )
{
// TODO: Implement OnPointDeleteAllClick
}

void PathAndPointManagerDialogImpl::OnPointPropertiesClick( wxCommandEvent& event )
{
// TODO: Implement OnPointPropertiesClick
}

void PathAndPointManagerDialogImpl::OnPointDefaultAction( wxListEvent& event )
{
// TODO: Implement OnPointDefaultAction
}

void PathAndPointManagerDialogImpl::OnPointNewClick( wxCommandEvent& event )
{
// TODO: Implement OnPointNewClick
}

void PathAndPointManagerDialogImpl::OnPointDeSelected( wxListEvent& event )
{
// TODO: Implement OnPointDeSelected
}