#ifndef __ODPropertiesDialogImpl__
#define __ODPropertiesDialogImpl__

/**
@file
Subclass of ODPropertiesDialogDef, which is generated by wxFormBuilder.
*/

#include "ODPropertiesDialogDef.h"

//// end generated include

/** Implementing ODPropertiesDialogDef */
class ODPropertiesDialogImpl : public ODPropertiesDialogDef
{
	protected:
		// Handlers for ODPropertiesDialogDef events.
		void OnDrawPropertiesOKClick( wxCommandEvent& event );
		void OnDrawPropertiesCancelClick( wxCommandEvent& event );
		void OnDrawPropertiesApplyClick( wxCommandEvent& event );
	public:
		/** Constructor */
		ODPropertiesDialogImpl( wxWindow* parent );
	//// end generated class members
	
};

#endif // __ODPropertiesDialogImpl__
