#ifndef __PathAndPointManagerDialogImpl__
#define __PathAndPointManagerDialogImpl__

/**
@file
Subclass of PathAndPointManagerDialogDef, which is generated by wxFormBuilder.
*/

#include "PathAndPointManagerDialogDef.h"

//// end generated include

/** Implementing PathAndPointManagerDialogDef */
class PathAndPointManagerDialogImpl : public PathAndPointManagerDialogDef
{
	protected:
		// Handlers for PathAndPointManagerDialogDef events.
		void OnTabSwitch( wxNotebookEvent& event );
		void OnPathToggleVisibility( wxMouseEvent& event );
		void OnPathColumnClicked( wxListEvent& event );
		void OnPathDefaultAction( wxListEvent& event );
		void OnPathDeSelected( wxListEvent& event );
		void OnPathRightClick( wxListEvent& event );
		void OnPathSelected( wxListEvent& event );
		void OnPathPropertiesClick( wxCommandEvent& event );
		void OnPathActivateClick( wxCommandEvent& event );
		void OnPathButtonLeftDown( wxMouseEvent& event );
		void OnPathCenterViewClick( wxCommandEvent& event );
		void OnPathDeleteClick( wxCommandEvent& event );
		void OnPathExportSelectedClick( wxCommandEvent& event );
		void OnPathDeleteAllClick( wxCommandEvent& event );
		void OnODPointToggleVisibility( wxMouseEvent& event );
		void OnODPointColumnClick( wxListEvent& event );
		void OnODPointDefaultAction( wxListEvent& event );
		void OnODPointDeSelected( wxListEvent& event );
		void OnODPointRightClick( wxListEvent& event );
		void OnODPointSelected( wxListEvent& event );
		void OnODPointNewClick( wxCommandEvent& event );
		void OnODPointPropertiesClick( wxCommandEvent& event );
		void OnODPointActivateClick( wxCommandEvent& event );
		void OnODPointCenterViewClick( wxCommandEvent& event );
		void OnODPointDeleteClick( wxCommandEvent& event );
		void OnODPointExportSecectedClick( wxCommandEvent& event );
		void OnODPointDeleteAllClick( wxCommandEvent& event );
		void OnLayerToggleVisibility( wxMouseEvent& event );
		void OnLayerColumnClick( wxListEvent& event );
		void OnLayerDefaultAction( wxListEvent& event );
		void OnLayerDeSelected( wxListEvent& event );
		void OnLayerRightClick( wxListEvent& event );
		void OnLayerSelected( wxListEvent& event );
		void OnLayerTemporaryClick( wxCommandEvent& event );
		void OnLayerDeleteClick( wxCommandEvent& event );
		void OnLayerShowOnChartClick( wxCommandEvent& event );
		void OnLayerShowPointNamesClick( wxCommandEvent& event );
		void OnLayerListContentsClick( wxCommandEvent& event );
		void OnImportClick( wxCommandEvent& event );
		void OnExportAllClick( wxCommandEvent& event );
		void OnOKClick( wxCommandEvent& event );
	public:
		/** Constructor */
		PathAndPointManagerDialogImpl( wxWindow* parent );
	//// end generated class members




};

#endif // __PathAndPointManagerDialogImpl__
